@@ -28,7 +28,6 @@ import pandas as pd
 from kubernetes import config as k8_config
 from kubernetes import watch as k8_watch
 from kubernetes.client import *
-from kubespawner.objects import make_pod, make_pvc
 from slugify import slugify
 
 from aws_orbit_sdk.common import get_properties, get_stepfunctions_waiter_config
@@ -50,6 +49,8 @@ MANIFEST_PROPERTY_MAP_TYPE = Dict[str, Union[str, Dict[str, Any]]]
 __CURRENT_TEAM_MANIFEST__: MANIFEST_TEAM_TYPE = None
 __CURRENT_ENV_MANIFEST__: MANIFEST_TEAM_TYPE = None
 
+APP_LABEL_SELECTOR: [str] = ["orbit-runner", "emr-spark"]
+
 
 def read_team_manifest_ssm(env_name: str, team_name: str) -> Optional[MANIFEST_TEAM_TYPE]:
     parameter_name: str = f"/orbit/{env_name}/teams/{team_name}/manifest"
@@ -316,33 +317,29 @@ def run_notebooks(taskConfiguration: dict) -> Any:
 
 
 def list_team_running_jobs():
-    return list_running_jobs(True)
+    props = get_properties()
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    return list_running_jobs(team_name)
 
 
 def list_my_running_jobs():
-    return list_running_jobs(False)
-
-
-def list_running_jobs(team_only: bool = False):
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
-    load_kube_config()
-    username = os.environ.get("JUPYTERHUB_USER", os.environ.get("USERNAME"))
-    api_instance = BatchV1Api()
-    # field_selector = "status.successful!=1"
-    if team_only:
-        operand = "!="
-    else:
-        operand = "="
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
+    return list_running_jobs(namespace)
 
-    label_selector = f"app=orbit-runner,username{operand}{username}"
-    _logger.info("using job selector %s", label_selector)
+
+def list_running_jobs(namespace: str):
+    props = get_properties()
+    load_kube_config()
+    api_instance = BatchV1Api()
+
+    label_selector = f"app=orbit-runner"
     try:
         api_response = api_instance.list_namespaced_job(
-            namespace=team_name,
+            namespace=namespace,
             _preload_content=False,
             label_selector=label_selector,
-            # field_selector=field_selector,
             watch=False,
         )
         res = json.loads(api_response.data)
@@ -356,14 +353,53 @@ def list_running_jobs(team_only: bool = False):
     return res["items"]
 
 
+def list_team_running_pods():
+    props = get_properties()
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    return list_running_pods(team_name)
+
+
+def list_my_running_pods():
+    props = get_properties()
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
+    return list_running_pods(namespace)
+
+
+def list_running_pods(namespace: str):
+    load_kube_config()
+    api_instance = CoreV1Api()
+
+    app_list = ",".join(APP_LABEL_SELECTOR)
+    label_selector = f"app in ({app_list})"
+    _logger.debug("using job selector %s", label_selector)
+    try:
+        api_response = api_instance.list_namespaced_pod(
+            namespace=namespace,
+            _preload_content=False,
+            label_selector=label_selector,
+            watch=False,
+        )
+        res = json.loads(api_response.data)
+    except ApiException as e:
+        _logger.info("Exception when calling CoreV1Api->list_namespaced_job: %s\n" % e)
+        raise e
+
+    if "items" not in res:
+        return []
+
+    return res["items"]
+
+
 def list_current_pods(label_selector: str = None):
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     load_kube_config()
     api_instance = CoreV1Api()
     try:
         params = dict()
-        params["namespace"] = team_name
+        params["namespace"] = namespace
         params["_preload_content"] = False
         if label_selector:
             params["label_selector"] = label_selector
@@ -383,10 +419,9 @@ def list_storage_pvc():
     load_kube_config()
     api_instance = CoreV1Api()
     props = get_properties()
-    team_name = props["AWS_ORBIT_TEAM_SPACE"]
-    _logger.debug(f"Listing {team_name} namespace persistent volume claims")
     params = dict()
-    params["namespace"] = team_name
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    params["namespace"] = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     params["_preload_content"] = False
     try:
         api_response = api_instance.list_namespaced_persistent_volume_claim(**params)
@@ -405,11 +440,10 @@ def delete_storage_pvc(pvc_name: str):
     load_kube_config()
     api_instance = CoreV1Api()
     props = get_properties()
-    team_name = props["AWS_ORBIT_TEAM_SPACE"]
-    _logger.debug(f"Deleting {team_name} namespace persistent volume claim {pvc_name}")
     params = dict()
     params["name"] = pvc_name
-    params["namespace"] = team_name
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    params["namespace"] = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     params["_preload_content"] = False
     try:
         api_response = api_instance.delete_namespaced_persistent_volume_claim(**params)
@@ -505,23 +539,41 @@ def get_nodegroups(cluster_name: str):
     return nodegroups_with_lt
 
 
+def delete_pod(pod_name: str, grace_period_seconds: int = 30):
+    props = get_properties()
+    global __CURRENT_TEAM_MANIFEST__, __CURRENT_ENV_MANIFEST__
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    load_kube_config()
+    api_instance = CoreV1Api()
+    try:
+        api_instance.delete_namespaced_pod(
+            name=pod_name,
+            namespace=os.environ.get("AWS_ORBIT_USER_SPACE", team_name),
+            _preload_content=False,
+            grace_period_seconds=grace_period_seconds,
+            orphan_dependents=False,
+        )
+    except ApiException as e:
+        _logger.info("Exception when calling CoreV1Api->delete_namespaced_pod: %s\n" % e)
+        raise e
+
+
 def delete_job(job_name: str, grace_period_seconds: int = 30):
     props = get_properties()
     global __CURRENT_TEAM_MANIFEST__, __CURRENT_ENV_MANIFEST__
-    env_name = props["AWS_ORBIT_ENV"]
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
     load_kube_config()
     api_instance = BatchV1Api()
     try:
         api_instance.delete_namespaced_job(
             name=job_name,
-            namespace=team_name,
+            namespace=os.environ.get("AWS_ORBIT_USER_SPACE", team_name),
             _preload_content=False,
             grace_period_seconds=grace_period_seconds,
             orphan_dependents=False,
         )
     except ApiException as e:
-        _logger.info("Exception when calling BatchV1Api->list_namespaced_job: %s\n" % e)
+        _logger.info("Exception when calling BatchV1Api->delete_namespaced_job: %s\n" % e)
         raise e
 
 
@@ -533,7 +585,7 @@ def delete_cronjob(job_name: str, grace_period_seconds: int = 30):
     try:
         api_instance.delete_namespaced_cron_job(
             name=job_name,
-            namespace=team_name,
+            namespace=os.environ.get("AWS_ORBIT_USER_SPACE", team_name),
             _preload_content=False,
             grace_period_seconds=grace_period_seconds,
             orphan_dependents=False,
@@ -543,16 +595,34 @@ def delete_cronjob(job_name: str, grace_period_seconds: int = 30):
         raise e
 
 
+def delete_all_my_pods():
+    props = get_properties()
+    team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
+    load_kube_config()
+    api_instance = CoreV1Api()
+    app_list = ",".join(APP_LABEL_SELECTOR)
+    label_selector = f"app in ({app_list})"
+    try:
+        api_instance.delete_collection_namespaced_pod(
+            namespace=namespace, _preload_content=False, orphan_dependents=False, label_selector=label_selector
+        )
+    except ApiException as e:
+        _logger.info("Exception when calling CoreV1Api->delete_collection_namespaced_pod: %s\n" % e)
+        raise e
+
+
 def delete_all_my_jobs():
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
-    username = os.environ.get("JUPYTERHUB_USER", os.environ.get("USERNAME"))
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
+
     load_kube_config()
     api_instance = BatchV1Api()
-    label_selector = f"app=orbit-runner,username={username}"
+    label_selector = f"app=orbit-runner"
     try:
         api_instance.delete_collection_namespaced_job(
-            namespace=team_name, _preload_content=False, orphan_dependents=False, label_selector=label_selector
+            namespace=namespace, _preload_content=False, orphan_dependents=False, label_selector=label_selector
         )
     except ApiException as e:
         _logger.info("Exception when calling BatchV1Api->delete_collection_namespaced_job: %s\n" % e)
@@ -562,10 +632,11 @@ def delete_all_my_jobs():
 def list_running_cronjobs():
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     load_kube_config()
     api_instance = BatchV1beta1Api()
     try:
-        api_response = api_instance.list_namespaced_cron_job(namespace=team_name, _preload_content=False)
+        api_response = api_instance.list_namespaced_cron_job(namespace=namespace, _preload_content=False)
         res = json.loads(api_response.data)
     except ApiException as e:
         _logger.info("Exception when calling BatchV1beta1Api->list_namespaced_cron_job: %s\n" % e)
@@ -577,7 +648,7 @@ def list_running_cronjobs():
     return res["items"]
 
 
-def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str], team_constants: TeamConstants) -> V1JobSpec:
+def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str]) -> V1JobSpec:
     """
     Runs Task in Python in a notebook using lambda.
 
@@ -599,14 +670,14 @@ def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str], team_c
     if __CURRENT_ENV_MANIFEST__ == None:
         __CURRENT_ENV_MANIFEST__ = load_env_context_from_ssm(env_name)
 
-    env = build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_constants, team_name)
-    profile = resolve_profile(taskConfiguration, team_constants)
-    image = resolve_image(__CURRENT_TEAM_MANIFEST__, profile)
+    env = build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_name)
+    profile = resolve_profile(taskConfiguration)
+    image = resolve_image(__CURRENT_ENV_MANIFEST__, profile)
     node_type = get_node_type(taskConfiguration)
 
     job_name: str = f'run-{taskConfiguration["task_type"]}'
-    volumes = team_constants.volumes()
-    volume_mounts = team_constants.volume_mounts()
+    volumes: List[Dict[str, Any]] = list()
+    volume_mounts: List[Dict[str, Any]] = list()
     grant_sudo = False
     if "kubespawner_override" in profile:
         if "volumes" in profile["kubespawner_override"]:
@@ -627,33 +698,27 @@ def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str], team_c
             _logger.info("task override is mounting volumes: %s", volume_mounts)
         if "labels" in taskConfiguration["compute"]:
             labels = {**labels, **taskConfiguration["compute"]["labels"]}
-    node_selector = team_constants.node_selector(node_type)
+    node_selector: List[Dict[str, str]] = list()
     _logger.info("volumes:%s", json.dumps(volumes))
     _logger.info("volume_mounts:%s", json.dumps(volume_mounts))
+
     pod_properties: Dict[str, str] = dict(
         name=job_name,
         image=image,
-        cmd=["bash", "-c", "/home/jovyan/.orbit/bootstrap.sh && python /opt/python-utils/notebook_cli.py"],
+        cmd=["bash", "-c", "python /opt/python-utils/notebook_cli.py"],
         port=22,
-        image_pull_policy=team_constants.image_pull_policy(),
-        image_pull_secrets=None,
+        service_account="default-editor",
         node_selector=node_selector,
-        run_as_uid=team_constants.uid(grant_sudo),
-        run_as_gid=team_constants.gid(),
-        fs_gid=team_constants.gid(),
         run_privileged=False,
         allow_privilege_escalation=True,
         env=env,
         volumes=volumes,
         volume_mounts=volume_mounts,
         labels=labels,
-        annotations=team_constants.annotations(),
-        lifecycle_hooks=team_constants.life_cycle_hooks(),
-        service_account=team_name,
         logger=_logger,
+        run_as_uid=1000,
+        run_as_gid=100,
     )
-    if grant_sudo:
-        pod_properties["uid"] = 0
 
     if "kubespawner_override" in profile:
         for k, v in profile["kubespawner_override"].items():
@@ -676,22 +741,434 @@ def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str], team_c
     return job_spec
 
 
-def resolve_image(__CURRENT_TEAM_MANIFEST__, profile):
+def make_pod(
+    name,
+    cmd,
+    port,
+    image,
+    image_pull_policy=None,
+    image_pull_secrets=None,
+    node_selector=None,
+    run_as_uid=None,
+    run_as_gid=None,
+    fs_gid=None,
+    supplemental_gids=None,
+    run_privileged=False,
+    allow_privilege_escalation=True,
+    env=None,
+    working_dir=None,
+    volumes=None,
+    volume_mounts=None,
+    labels=None,
+    annotations=None,
+    cpu_limit=None,
+    cpu_guarantee=None,
+    mem_limit=None,
+    mem_guarantee=None,
+    extra_resource_limits=None,
+    extra_resource_guarantees=None,
+    lifecycle_hooks=None,
+    init_containers=None,
+    service_account=None,
+    extra_container_config=None,
+    extra_pod_config=None,
+    extra_containers=None,
+    scheduler_name=None,
+    tolerations=None,
+    node_affinity_preferred=None,
+    node_affinity_required=None,
+    pod_affinity_preferred=None,
+    pod_affinity_required=None,
+    pod_anti_affinity_preferred=None,
+    pod_anti_affinity_required=None,
+    priority_class_name=None,
+    logger=None,
+):
+    """
+    Make a k8s pod specification for running a user notebook.
+
+    Parameters
+    ----------
+    name:
+        Name of pod. Must be unique within the namespace the object is
+        going to be created in. Must be a valid DNS label.
+    image:
+        Image specification - usually a image name and tag in the form
+        of image_name:tag. Same thing you would use with docker commandline
+        arguments
+    image_pull_policy:
+        Image pull policy - one of 'Always', 'IfNotPresent' or 'Never'. Decides
+        when kubernetes will check for a newer version of image and pull it when
+        running a pod.
+    image_pull_secrets:
+        Image pull secrets - a list of references to Kubernetes Secret resources
+        with credentials to pull images from image registries. This list can
+        either have strings in it or objects with the string value nested under
+        a name field.
+    port:
+        Port the notebook server is going to be listening on
+    cmd:
+        The command used to execute the singleuser server.
+    node_selector:
+        Dictionary Selector to match nodes where to launch the Pods
+    run_as_uid:
+        The UID used to run single-user pods. The default is to run as the user
+        specified in the Dockerfile, if this is set to None.
+    run_as_gid:
+        The GID used to run single-user pods. The default is to run as the primary
+        group of the user specified in the Dockerfile, if this is set to None.
+        Setting this parameter requires that *feature-gate* **RunAsGroup** be enabled,
+        otherwise the effective GID of the pod will be 0 (root).  In addition, not
+        setting `run_as_gid` once feature-gate RunAsGroup is enabled will also
+        result in an effective GID of 0 (root).
+    fs_gid
+        The gid that will own any fresh volumes mounted into this pod, if using
+        volume types that support this (such as GCE). This should be a group that
+        the uid the process is running as should be a member of, so that it can
+        read / write to the volumes mounted.
+    supplemental_gids:
+        A list of GIDs that should be set as additional supplemental groups to
+        the user that the container runs as. You may have to set this if you are
+        deploying to an environment with RBAC/SCC enforced and pods run with a
+        'restricted' SCC which results in the image being run as an assigned
+        user ID. The supplemental group IDs would need to include the
+        corresponding group ID of the user ID the image normally would run as.
+        The image must setup all directories/files any application needs access
+        to, as group writable.
+    run_privileged:
+        Whether the container should be run in privileged mode.
+    allow_privilege_escalation:
+        Controls whether a process can gain more privileges than its parent process.
+    env:
+        Dictionary of environment variables.
+    volumes:
+        List of dictionaries containing the volumes of various types this pod
+        will be using. See k8s documentation about volumes on how to specify
+        these
+    volume_mounts:
+        List of dictionaries mapping paths in the container and the volume(
+        specified in volumes) that should be mounted on them. See the k8s
+        documentaiton for more details
+    working_dir:
+        String specifying the working directory for the notebook container
+    labels:
+        Labels to add to the spawned pod.
+    annotations:
+        Annotations to add to the spawned pod.
+    cpu_limit:
+        Float specifying the max number of CPU cores the user's pod is
+        allowed to use.
+    cpu_guarentee:
+        Float specifying the max number of CPU cores the user's pod is
+        guaranteed to have access to, by the scheduler.
+    mem_limit:
+        String specifying the max amount of RAM the user's pod is allowed
+        to use. String instead of float/int since common suffixes are allowed
+    mem_guarantee:
+        String specifying the max amount of RAM the user's pod is guaranteed
+        to have access to. String ins loat/int since common suffixes
+        are allowed
+    lifecycle_hooks:
+        Dictionary of lifecycle hooks
+    init_containers:
+        List of initialization containers belonging to the pod.
+    service_account:
+        Service account to mount on the pod. None disables mounting
+    extra_container_config:
+        Extra configuration (e.g. envFrom) for notebook container which is not covered by parameters above.
+    extra_pod_config:
+        Extra configuration (e.g. tolerations) for pod which is not covered by parameters above.
+    extra_containers:
+        Extra containers besides notebook container. Used for some housekeeping jobs (e.g. crontab).
+    scheduler_name:
+        The pod's scheduler explicitly named.
+    tolerations:
+        Tolerations can allow a pod to schedule or execute on a tainted node. To
+        learn more about pod tolerations, see
+        https://kubernetes.io/docs/concepts/configuration/taint-and-toleration/.
+
+        Pass this field an array of "Toleration" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#nodeselectorterm-v1-core
+    node_affinity_preferred:
+        Affinities describe where pods prefer or require to be scheduled, they
+        may prefer or require a node to have a certain label or be in proximity
+        / remoteness to another pod. To learn more visit
+        https://kubernetes.io/docs/concepts/configuration/assign-pod-node/
+
+        Pass this field an array of "PreferredSchedulingTerm" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#preferredschedulingterm-v1-core
+    node_affinity_required:
+        Affinities describe where pods prefer or require to be scheduled, they
+        may prefer or require a node to have a certain label or be in proximity
+        / remoteness to another pod. To learn more visit
+        https://kubernetes.io/docs/concepts/configuration/assign-pod-node/
+
+        Pass this field an array of "NodeSelectorTerm" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#nodeselectorterm-v1-core
+    pod_affinity_preferred:
+        Affinities describe where pods prefer or require to be scheduled, they
+        may prefer or require a node to have a certain label or be in proximity
+        / remoteness to another pod. To learn more visit
+        https://kubernetes.io/docs/concepts/configuration/assign-pod-node/
+
+        Pass this field an array of "WeightedPodAffinityTerm" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#weightedpodaffinityterm-v1-core
+    pod_affinity_required:
+        Affinities describe where pods prefer or require to be scheduled, they
+        may prefer or require a node to have a certain label or be in proximity
+        / remoteness to another pod. To learn more visit
+        https://kubernetes.io/docs/concepts/configuration/assign-pod-node/
+
+        Pass this field an array of "PodAffinityTerm" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#podaffinityterm-v1-core
+    pod_anti_affinity_preferred:
+        Affinities describe where pods prefer or require to be scheduled, they
+        may prefer or require a node to have a certain label or be in proximity
+        / remoteness to another pod. To learn more visit
+        https://kubernetes.io/docs/concepts/configuration/assign-pod-node/
+
+        Pass this field an array of "WeightedPodAffinityTerm" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#weightedpodaffinityterm-v1-core
+    pod_anti_affinity_required:
+        Affinities describe where pods prefer or require to be scheduled, they
+        may prefer or require a node to have a certain label or be in proximity
+        / remoteness to another pod. To learn more visit
+        https://kubernetes.io/docs/concepts/configuration/assign-pod-node/
+
+        Pass this field an array of "PodAffinityTerm" objects.*
+        * https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#podaffinityterm-v1-core
+    priority_class_name:
+        The name of the PriorityClass to be assigned the pod. This feature is Beta available in K8s 1.11.
+    """
+
+    pod = V1Pod()
+    pod.kind = "Pod"
+    pod.api_version = "v1"
+
+    pod.metadata = V1ObjectMeta(name=name, labels=(labels or {}).copy(), annotations=(annotations or {}).copy())
+
+    pod.spec = V1PodSpec(containers=[])
+    pod.spec.restart_policy = "OnFailure"
+
+    if image_pull_secrets is not None:
+        # image_pull_secrets as received by the make_pod function should always
+        # be a list, but it is allowed to have "a-string" elements or {"name":
+        # "a-string"} elements.
+        pod.spec.image_pull_secrets = [
+            V1LocalObjectReference(name=secret_ref)
+            if type(secret_ref) == str
+            else get_k8s_model(V1LocalObjectReference, secret_ref)
+            for secret_ref in image_pull_secrets
+        ]
+
+    if node_selector:
+        pod.spec.node_selector = node_selector
+
+    if lifecycle_hooks:
+        lifecycle_hooks = get_k8s_model(V1Lifecycle, lifecycle_hooks)
+
+    # There are security contexts both on the Pod level or the Container level.
+    # The security settings that you specify for a Pod apply to all Containers
+    # in the Pod, but settings on the container level can override them.
+    #
+    # We configure the pod to be spawned on the container level unless the
+    # option is only available on the pod level, such as for those relating to
+    # the volumes as compared to the running user of the container. Volumes
+    # belong to the pod and are only mounted by containers after all.
+    #
+    # ref: https://kubernetes.io/docs/tasks/configure-pod-container/security-context/
+    # ref: https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.16/#securitycontext-v1-core (container)
+    # ref: https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.16/#podsecuritycontext-v1-core (pod)
+    pod_security_context = V1PodSecurityContext()
+    if fs_gid is not None:
+        pod_security_context.fs_group = int(fs_gid)
+    if supplemental_gids is not None and supplemental_gids:
+        pod_security_context.supplemental_groups = [int(gid) for gid in supplemental_gids]
+    # Only clutter pod spec with actual content
+    if not all([e is None for e in pod_security_context.to_dict().values()]):
+        pod.spec.security_context = pod_security_context
+
+    container_security_context = V1SecurityContext()
+    if run_as_uid is not None:
+        container_security_context.run_as_user = int(run_as_uid)
+    if run_as_gid is not None:
+        container_security_context.run_as_group = int(run_as_gid)
+    if run_privileged:
+        container_security_context.privileged = True
+    if not allow_privilege_escalation:
+        container_security_context.allow_privilege_escalation = False
+    # Only clutter container spec with actual content
+    if all([e is None for e in container_security_context.to_dict().values()]):
+        container_security_context = None
+
+    # Transform a dict into valid Kubernetes EnvVar Python representations. This
+    # representation shall always have a "name" field as well as either a
+    # "value" field or "value_from" field. For examples see the
+    # test_make_pod_with_env function.
+    prepared_env = []
+    for k, v in (env or {}).items():
+        if type(v) == dict:
+            if not "name" in v:
+                v["name"] = k
+            prepared_env.append(get_k8s_model(V1EnvVar, v))
+        else:
+            prepared_env.append(V1EnvVar(name=k, value=v))
+    notebook_container = V1Container(
+        name="orbit-runner",
+        image=image,
+        working_dir=working_dir,
+        ports=[V1ContainerPort(name="notebook-port", container_port=port)],
+        env=prepared_env,
+        args=cmd,
+        image_pull_policy=image_pull_policy,
+        lifecycle=lifecycle_hooks,
+        resources=V1ResourceRequirements(),
+        volume_mounts=[get_k8s_model(V1VolumeMount, obj) for obj in (volume_mounts or [])],
+        security_context=container_security_context,
+    )
+
+    if service_account is None:
+        # This makes sure that we don't accidentally give access to the whole
+        # kubernetes API to the users in the spawned pods.
+        pod.spec.automount_service_account_token = False
+    else:
+        pod.spec.service_account_name = service_account
+
+    notebook_container.resources.requests = {}
+    if cpu_guarantee:
+        notebook_container.resources.requests["cpu"] = cpu_guarantee
+    if mem_guarantee:
+        notebook_container.resources.requests["memory"] = mem_guarantee
+    if extra_resource_guarantees:
+        notebook_container.resources.requests.update(extra_resource_guarantees)
+
+    notebook_container.resources.limits = {}
+    if cpu_limit:
+        notebook_container.resources.limits["cpu"] = cpu_limit
+    if mem_limit:
+        notebook_container.resources.limits["memory"] = mem_limit
+    if extra_resource_limits:
+        notebook_container.resources.limits.update(extra_resource_limits)
+
+    if extra_container_config:
+        notebook_container = update_k8s_model(
+            target=notebook_container,
+            changes=extra_container_config,
+            logger=logger,
+            target_name="notebook_container",
+            changes_name="extra_container_config",
+        )
+
+    pod.spec.containers.append(notebook_container)
+
+    if extra_containers:
+        pod.spec.containers.extend([get_k8s_model(V1Container, obj) for obj in extra_containers])
+    if tolerations:
+        pod.spec.tolerations = [get_k8s_model(V1Toleration, obj) for obj in tolerations]
+    if init_containers:
+        pod.spec.init_containers = [get_k8s_model(V1Container, obj) for obj in init_containers]
+    if volumes:
+        pod.spec.volumes = [get_k8s_model(V1Volume, obj) for obj in volumes]
+    else:
+        # Keep behaving exactly like before by not cleaning up generated pod
+        # spec by setting the volumes field even though it is an empty list.
+        pod.spec.volumes = []
+    if scheduler_name:
+        pod.spec.scheduler_name = scheduler_name
+
+    node_affinity = None
+    if node_affinity_preferred or node_affinity_required:
+        node_selector = None
+        if node_affinity_required:
+            node_selector = V1NodeSelector(
+                node_selector_terms=[get_k8s_model(V1NodeSelectorTerm, obj) for obj in node_affinity_required],
+            )
+
+        preferred_scheduling_terms = None
+        if node_affinity_preferred:
+            preferred_scheduling_terms = [
+                get_k8s_model(V1PreferredSchedulingTerm, obj) for obj in node_affinity_preferred
+            ]
+
+        node_affinity = V1NodeAffinity(
+            preferred_during_scheduling_ignored_during_execution=preferred_scheduling_terms,
+            required_during_scheduling_ignored_during_execution=node_selector,
+        )
+
+    pod_affinity = None
+    if pod_affinity_preferred or pod_affinity_required:
+        weighted_pod_affinity_terms = None
+        if pod_affinity_preferred:
+            weighted_pod_affinity_terms = [
+                get_k8s_model(V1WeightedPodAffinityTerm, obj) for obj in pod_affinity_preferred
+            ]
+
+        pod_affinity_terms = None
+        if pod_affinity_required:
+            pod_affinity_terms = [get_k8s_model(V1PodAffinityTerm, obj) for obj in pod_affinity_required]
+
+        pod_affinity = V1PodAffinity(
+            preferred_during_scheduling_ignored_during_execution=weighted_pod_affinity_terms,
+            required_during_scheduling_ignored_during_execution=pod_affinity_terms,
+        )
+
+    pod_anti_affinity = None
+    if pod_anti_affinity_preferred or pod_anti_affinity_required:
+        weighted_pod_affinity_terms = None
+        if pod_anti_affinity_preferred:
+            weighted_pod_affinity_terms = [
+                get_k8s_model(V1WeightedPodAffinityTerm, obj) for obj in pod_anti_affinity_preferred
+            ]
+
+        pod_affinity_terms = None
+        if pod_anti_affinity_required:
+            pod_affinity_terms = [get_k8s_model(V1PodAffinityTerm, obj) for obj in pod_anti_affinity_required]
+
+        pod_anti_affinity = V1PodAffinity(
+            preferred_during_scheduling_ignored_during_execution=weighted_pod_affinity_terms,
+            required_during_scheduling_ignored_during_execution=pod_affinity_terms,
+        )
+
+    affinity = None
+    if node_affinity or pod_affinity or pod_anti_affinity:
+        affinity = V1Affinity(
+            node_affinity=node_affinity,
+            pod_affinity=pod_affinity,
+            pod_anti_affinity=pod_anti_affinity,
+        )
+
+    if affinity:
+        pod.spec.affinity = affinity
+
+    if priority_class_name:
+        pod.spec.priority_class_name = priority_class_name
+
+    if extra_pod_config:
+        pod.spec = update_k8s_model(
+            target=pod.spec,
+            changes=extra_pod_config,
+            logger=logger,
+            target_name="pod.spec",
+            changes_name="extra_pod_config",
+        )
+
+    return pod
+
+
+def resolve_image(__CURRENT_ENV_MANIFEST__, profile):
     if not profile or "kubespawner_override" not in profile or "image" not in profile["kubespawner_override"]:
-        repository = __CURRENT_TEAM_MANIFEST__["FinalImageAddress"]
+        repository = (
+            f'{__CURRENT_ENV_MANIFEST__["Images"]["JupyterUser"]["Repository"]}:'
+            f'{__CURRENT_ENV_MANIFEST__["Images"]["JupyterUser"]["Version"]}'
+        )
         image = f"{repository}"
     else:
         image = profile["kubespawner_override"]["image"]
     return image
 
 
-def build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_constants, team_name):
-    env = team_constants.env()
-    env["AWS_ORBIT_ENV"] = env_name
-    env["AWS_ORBIT_TEAM_SPACE"] = team_name
-    env["JUPYTERHUB_USER"] = team_constants.username
-    env["USERNAME"] = team_constants.username
-    env["AWS_ORBIT_S3_BUCKET"] = __CURRENT_ENV_MANIFEST__["Toolkit"]["S3Bucket"]
+def build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_name):
+    env = dict()
     env["task_type"] = taskConfiguration["task_type"]
     env["tasks"] = json.dumps({"tasks": taskConfiguration["tasks"]})
     if "compute" in taskConfiguration:
@@ -701,7 +1178,8 @@ def build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_consta
     return env
 
 
-def resolve_profile(taskConfiguration, team_constants):
+def resolve_profile(taskConfiguration):
+    team_constants = TeamConstants()
     if "compute" in taskConfiguration and "profile" in taskConfiguration["compute"]:
         profile_name = taskConfiguration["compute"]["profile"]
         profile_name = slugify(profile_name)
@@ -730,13 +1208,12 @@ def _run_task_eks(taskConfiguration: dict) -> Any:
     """
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
-    username = os.environ.get("JUPYTERHUB_USER", os.environ.get("USERNAME"))
+    username = (os.environ.get("JUPYTERHUB_USER", os.environ.get("USERNAME"))).split("@")[0]
     node_type = get_node_type(taskConfiguration)
-    labels = {"app": f"orbit-runner", "orbit/node-type": node_type, "username": username}
+    labels = {"app": f"orbit-runner", "orbit/node-type": node_type, "notebook-name": os.environ.get("HOSTNAME")}
     if node_type == "ec2":
         labels["orbit/attach-security-group"] = "yes"
-    team_constants: TeamConstants = TeamConstants()
-    job_spec = _create_eks_job_spec(taskConfiguration, labels=labels, team_constants=team_constants)
+    job_spec = _create_eks_job_spec(taskConfiguration, labels=labels)
     load_kube_config()
     if "compute" in taskConfiguration:
         if "labels" in taskConfiguration["compute"]:
@@ -745,12 +1222,14 @@ def _run_task_eks(taskConfiguration: dict) -> Any:
         api_version="batch/v1",
         kind="Job",
         metadata=V1ObjectMeta(
-            generate_name=f"orbit-{team_name}-{node_type}-runner-", labels=labels, namespace=team_name
+            generate_name=f"orbit-{team_name}-{node_type}-runner-",
+            labels=labels,
+            namespace=os.environ.get("AWS_ORBIT_USER_SPACE", team_name),
         ),
         spec=job_spec,
     )
     job_instance: V1Job = BatchV1Api().create_namespaced_job(
-        namespace=team_name,
+        namespace=os.environ.get("AWS_ORBIT_USER_SPACE", team_name),
         body=job,
     )
     metadata: V1ObjectMeta = job_instance.metadata
@@ -885,24 +1364,30 @@ def schedule_task_eks(triggerName: str, frequency: str, taskConfiguration: dict)
     """
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     node_type = get_node_type(taskConfiguration)
-    username = os.environ.get("JUPYTERHUB_USER", os.environ.get("USERNAME"))
-    cronjob_id = f"orbit-{team_name}-{triggerName}"
-    labels = {"app": f"orbit-runner", "orbit/node-type": node_type, "username": username, "cronjob_id": cronjob_id}
-    team_constants: TeamConstants = TeamConstants(username)
-    job_spec = _create_eks_job_spec(taskConfiguration, labels=labels, team_constants=team_constants)
+    username = (os.environ.get("JUPYTERHUB_USER", os.environ.get("USERNAME"))).split("@")[0]
+    cronjob_id = f"orbit-{namespace}-{triggerName}"
+    labels = {
+        "app": f"orbit-runner",
+        "orbit/node-type": node_type,
+        "cronjob_id": cronjob_id,
+        "notebook-name": "scheduled",
+    }
+
+    job_spec = _create_eks_job_spec(taskConfiguration, labels=labels)
     cron_job_template: V1beta1JobTemplateSpec = V1beta1JobTemplateSpec(spec=job_spec)
     cron_job_spec: V1beta1CronJobSpec = V1beta1CronJobSpec(job_template=cron_job_template, schedule=frequency)
     job = V1beta1CronJob(
         api_version="batch/v1beta1",
         kind="CronJob",
-        metadata=V1ObjectMeta(name=cronjob_id, labels=labels, namespace=team_name),
+        metadata=V1ObjectMeta(name=cronjob_id, labels=labels, namespace=namespace),
         status=V1beta1CronJobStatus(),
         spec=cron_job_spec,
     )
     load_kube_config()
 
-    job_instance: V1beta1CronJob = BatchV1beta1Api().create_namespaced_cron_job(namespace=team_name, body=job)
+    job_instance: V1beta1CronJob = BatchV1beta1Api().create_namespaced_cron_job(namespace=namespace, body=job)
     metadata: V1ObjectMeta = job_instance.metadata
     return {
         "ExecutionType": "eks",
@@ -929,9 +1414,10 @@ def delete_task_schedule(triggerName: str, compute_type: str = "eks") -> None:
 def delete_task_schedule_eks(triggerName: str) -> None:
     props = get_properties()
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     load_kube_config()
 
-    BatchV1beta1Api().delete_namespaced_cron_job(name=f"orbit-{team_name}-{triggerName}", namespace=team_name)
+    BatchV1beta1Api().delete_namespaced_cron_job(name=f"orbit-{team_name}-{triggerName}", namespace=namespace)
 
 
 def order_def(task_definition: Any) -> datetime:
@@ -989,21 +1475,22 @@ def wait_for_tasks_to_complete(
     team_name = props["AWS_ORBIT_TEAM_SPACE"]
 
     incomplete_tasks = []
-    _logger.info("Waiting2 for %s tasks %s", len(tasks), tasks)
+    _logger.info("Waiting for %s tasks %s", len(tasks), tasks)
     load_kube_config()
+    namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
     while True:
         for task in tasks:
             _logger.debug("Checking execution state of: %s", task)
             try:
                 current_jobs: V1JobList = BatchV1Api().list_namespaced_job(
-                    namespace=team_name, label_selector=f"app=orbit-runner"
+                    namespace=namespace, label_selector=f"app=orbit-runner"
                 )
             except exceptions.ApiException as e:
                 _logger.error("Error during list jobs for %s: %s", team_name, e)
                 # try again after 5 seconds.
                 time.sleep(5)
                 current_jobs: V1JobList = BatchV1Api().list_namespaced_job(
-                    namespace=team_name, label_selector=f"app=orbit-runner"
+                    namespace=namespace, label_selector=f"app=orbit-runner"
                 )
             task_name = task["Identifier"]
             for job in current_jobs.items:
@@ -1048,9 +1535,9 @@ def tail_logs(team_name, tasks) -> None:
     for task in tasks:
         task_id = task["Identifier"]
         _logger.info("Watching task: '%s'", task_id)
-
+        namespace = os.environ.get("AWS_ORBIT_USER_SPACE", team_name)
         current_pods: V1PodList = CoreV1Api().list_namespaced_pod(
-            namespace=team_name, label_selector=f"job-name={task_id}"
+            namespace=namespace, label_selector=f"job-name={task_id}"
         )
         for pod in current_pods.items:
             pod_instance: V1Pod = cast(V1Pod, pod)
@@ -1072,7 +1559,7 @@ def tail_logs(team_name, tasks) -> None:
                         _logger.info("task %s status: %s", pod_instance.metadata.name, container_state)
                         w = k8_watch.Watch()
                         for line in w.stream(
-                            CoreV1Api().read_namespaced_pod_log, name=pod_instance.metadata.name, namespace=team_name
+                            CoreV1Api().read_namespaced_pod_log, name=pod_instance.metadata.name, namespace=namespace
                         ):
                             _logger.info(line)
                     else:
